/*----------------------------------------------------------------------------
 *      Name:    Serial.h
 *      Purpose: Definitions and config for Uart
 */

#define COMM_0			0
#define COMM_1			1
#define USB_PORT		2
#define USB_PORT_INIT   3

#define CR     0x0D

#define BUFFSIZE 256

#define MAX_SERIAL_BUF_SIZE	0x40

#define MAX_TIMEOUT_USB				5
#define MAX_USB_TIMEOUT_IN_TICKS	800 //was 200												   


//UART Status Register Values
#define RX_DATA_RDY 0x1
#define OVERRUN_ERR 0x2
#define PARITY_ERROR 0x4
#define FRAMING_ERROR 0x8
#define THR_EMPTY 0x20
#define TX_EMPTY 0x40
#define RX_ERROR 0x80
#define ERROR_STATUS 0xF

#define INTIDMASK 0x0E

// UART Interrupt Identification Reg
#define RX_INT_STAT 0x9E
#define RX_LINE_STATUS 0x06
#define RX_DATA_AVAIL 0x04
#define CHAR_TIMEOUT 0x0C
#define THRE_INTERRUPT 0x02
#define INT_PENDING 0x01    
#define PARITY_ERR_MASK 0x00000100  

#define UART1_INT 1	 			//Enable uart 1 interrupt//
#define UART0_INT 1	 			//Enable uart 0 interrupt
//#define UART1_TX_INT 1	//Enable uart 1 THRE interrupt//

#define TX_DISABLE 0x0 			//Uart disable transmission
#define TX_ENABLE 0x8				//Uart enable transmission
#define RTS_0PIN 0x1000
#define NOT_READY_TO_SEND 1
#define READY_TO_SEND 0

//External interrupt selections
#define FALLING_EDGE	 0
#define RISING_EDGE 1
#define EDGE_SELECT_MODE 0x04


//Variables
//unsigned int packetNumber;
extern unsigned long packetNumber;
extern int rx1Buffer[BUFFSIZE];
extern int tx1Buffer[BUFFSIZE];
extern long rdRxBuff1, wrRxBuff1;
extern long rdTxBuff1, wrTxBuff1;
extern uchar mySerial0In[];
extern uchar mySerial0Pos;

//Serial API Calls
extern void putString(uchar myPort, char *);
extern int getkey (uchar);
extern int sendchar (uchar, int ch);
void uart0_isr (void)__irq; 
void uart1_isr (void)__irq; 
extern void init_serial0 (void);
extern void init_serial1 (void);
extern void resetAllRxBuffers(void);
extern void resetUART0RxBuffers(void);
extern void resetUART1RxBuffers(void);
extern long putcharComm0 (long ch);
extern long getcharComm0 (void);
extern void setComm0RTS(long rtsValue);
extern int putcharComm1 (int ch);
extern long putcharComm1Setup (uchar ch);
extern short getCharComm1 (char* myChar);
extern void diagSerial(void);
extern void sendStr0(const char * strBuffer0);
extern void sendStr1(const char * strBuffer1);
extern long getStr0(char * strBuffer);
extern long getStr1(char * strBuffer);
void ResetAndSuspendUart0(void);
void ResumeUart0(void);
extern void txTest(void);
extern void clrUart1RxBuffer(void);

extern long strToNum(int * string);
extern void putdecimal(unsigned long v);

extern void diagSerialSP(void);
extern void diagTXButton(void);
extern void diagEchoButton(void);
extern void setParityStatus(long statusVal);
extern void selParity(long parityType);
extern void selTHRIntr(long THRVal);
extern void txUart1Buffer(void);
extern ulong chkTXUart1Buffer(void);

//Initialization call
extern void serialInit(void);
