//
//						system.h
//
//	Include file that contains system related literals
//
//	(C) SoftwareFlair Inc. 2007
//
//
//
//	Hardware:
//

#define RELEASE_MODE		1

#ifdef RELEASE_MODE
#define VORT_BOARD	 		1   // normally un commented to build for the Laser board, and not the Keil Development board MBC2148
#endif

//#define SW_DEBUG			1	// normally commented out, used for the debug builds
//#define SIMULATOR_ACTIVE	1


#define SW_VERSION						"2.0.1Y"
#define SW_PROTOCOL_VERSION		"1.0.0"
#define SYS_CFG_VERSION				1


#define STREAM_SERIAL					0x01
#define STREAM_USB						0x02

#define MAX_CMD_SIZE					80
#define MAX_TIMEOUT_A2D				5

#define BOARD_ID_X1		0x0		// original X1 Board


// allows soome calculations to provide, or not provide, parsing responses
#define VORT_PARSER	1
#define NO_PARSER		0

#define STATUS_LED_RUNNING 1
#define STATUS_LED_DEBUG 0


//Compiler doesn't accept these in older version
typedef	unsigned char 	uchar;
typedef	unsigned short 	ushort;
typedef	unsigned long 	ulong;

// Stop Watch Type: used to keep track of laser time on
typedef enum {
	RESET = 0,
	STOP,
	START,
	LOG,
	TRIGGER
} STOP_WATCH_TYPE;

typedef enum {
	EXT_A2D_LSR_CUR = 0, 	// channel 0
	EXT_A2D_LSR_PWR,
	EXT_A2D_LSR_DRV_CUR
} EXT_A2D_TYPE;

typedef enum {
	EXT_A2D_SOA_CURR=0,
	EXT_A2D_DFB_CURR,
	EXT_A2D_HEATER_CURR	
} EXT_A2D_CURRENT_TYPE;

// Elapsed time: stop watch for - laser on, and command timeout from comm ports
typedef enum
{
	LSR_STOP_WATCH = 0x0,
	CMD_COMM0_STOP_WATCH,
	CMD_COMM1_STOP_WATCH,
	CMD_USB_STOP_WATCH,
	INIT_CURR_PWRUP_STOP_WATCH,
	STOP_WATCH_MAX_SEL				// Leave at the bottom of the enumeration!!!
} STOP_WATCH_SEL;

#define HIGH_LVL	1
#define LOW_LVL		0


#define PASS		0
#define FAIL		-1

#define INVALID_ERR	-1

#define A2D_CHNL_COLD_BLOCK			1
#define A2D_CHNL_BASE_PLATE			2

// Hardware Port 0 I/O pin assignment
#define PIN_SCL0									0x00000004		// P0.2 I2C SCL0 - clock (Output)
#define PIN_BOARD_ID_0						0x00000010		// P0.4 Board ID LSBit (Input)
#define PIN_BOARD_ID_1						0x00000020		// P0.5 Board ID (Input)
#define PIN_BOARD_ID_2						0x00000040		// P0.6 Board ID MSBit (Input)
#define PIN_BOARD_ID_CTRL					0x00000080		// P0.7 Controls output to set inputs P0.4 - P0.6  (OUTPUT)

#define PIN_EXT_INTERLOCK					0x00000400		// P0.10 - INTERLOCK "IN"	External Inerlock input
#define PIN_LSR_EN								0x00000800		// P0.11 - LASER ENABLE	"IN"	External Laser ON-OFF
#define PIN_AUTO_START						0x00001000		// P0.12 - AUTOSTART "IN"	External Autostart on-off
#define PIN_DEBUG_STROBE					0x00002000		// P0.13 - Debug pin for performance analysis, ...
#define PIN_LED_STATUS						0x00008000		// P0.15 - LED Running Status
//#define PIN_TE_ON									0x00010000		// P0.16 - TE ON "OUT"	Controls TE Cooler ON-OFF			 SWAPPED based on Board Revision (P0.16 and P0.19)!!

#define PIN_FAULT_INDC						    0x00020000	// P0.17 - FAULT INDICATOR "OUT"	External Fault indicator
#define PIN_LSR_RDY								0x00040000	// P0.18 - LASER READY "OUT"	External Laser Ready line

//#define PIN_I_MOD_ON							0x00080000	// P0.19 - I MODULATION ON "OUT"	Turns External Modulation of the laser Current  ON-OFF	 SWAPPED based on Board Revision (P0.16 and P0.19)!!
//#define PIN_MOD_ON								0x00080000	// P0.19 - MODULATION ON "OUT" External Modulation of the laser   ON-OFF
#define PIN_LIGHT_MOD_ON					0x00100000	// P0.20 - LIGHT MODULATION ON "OUT"	Turns External Modulation of the laser light ON-OFF
#define PIN_LIGHT_CUR_MOD_ON			0x00100000	// P0.20 - LIGHT OR CURRENT MODULATION ON "OUT"	Turns External Modulation of the laser light or current ON-OFF

#define PIN_LSR_ON								0x00200000	// P0.21 - LASER OK "OUT"	Microprocessor control of laser light
#define PIN_LSR_EMISSION_LED			0x00400000	// P0.22 - LASER EMISSION "OUT"	Indicates Laser Emission likely


#define PIN_COLD_BLOCK_TEMP_ADJ		0x02000000	// P0.25 - COLD BLOCK TEMP ADJ, (DAC0) "ANALOG OUT"	Sets Cold Block Temperature
#define PIN_COLD_BLOCK_TEMP				0x10000000	// P0.28 - COLD BLOCK TEMP, (ADC0.1)	"ANALOG IN"	Reads Cold Block Temperature
#define PIN_BASE_PLATE_TEMP				0x20000000	// P0.29 - BASE PLATE TEMP, (ADC0.2)	"ANALOG IN"	Read Base Plate Temperature
#define PIN_USB_RENUM							0x80000000	// P0.31 - Output to renumerate USB

extern short isUsbOutDataAvailable;
extern char myUsbOutBuf[64];
extern char myUsbInBuf[64];
extern char myUsbInTmpBuf[64];
extern char myUsbInTmpBuf2[64];
extern volatile uchar UsbInPending;

extern ulong PIN_TE_ON;
extern ulong PIN_I_MOD_ON;
extern ulong PIN_MOD_ON;

extern ushort isGuiCommunicatingToVort;
extern char isAutoStartActive;
extern short cdrhTriggersLsr;
extern ulong cdrhStartTime;
extern ulong execTicks;		// Exec Tick 10ms resolution, fixed at 10ms
extern ulong execTicksDebug;
extern ulong oneSecTicks;
extern ulong debugTicks;
extern ulong swTicks;		// sw Tick 10ms resolution, period is specified by software
extern uchar isLsrEnableActive;
extern ushort extA2dValues[3];
extern ushort extA2dCurrentValues[3];
extern ushort extA2dDfbValues[3];
extern uchar currentLe;
extern short wasHere;


extern char comm0RxBuf[];
extern char comm0RxInPos;
extern char comm0RxOutPos;
extern uchar charsToProcessRx0;
extern char rx0Buf[];

extern int prevWDogOccurred;

// Laser Monitor modes
extern short monCdrhHandlerMode;
extern short monStdbHandlerMode;
extern short monIntlHandlerMode;
extern short monDiCuHandlerMode;
extern short monDTmpHandlerMode;
extern short monBPltHandlerMode;

extern uchar isA2d0ConvDone;
extern const long vort2BrandX[];

extern void initBoard(void);
extern void brownOutInit(void);
extern ushort chkSumCalc(ushort myStartAddr, ushort myEndAddr);
extern char didBrownOutOccur(void);
extern void lsrOnStopWatch(STOP_WATCH_SEL mySel, STOP_WATCH_TYPE myMode);

extern void swError(char* myFile, short myLine);
extern void swTrace(char* myFile, short myLine);
extern void DelayUs(ushort);
extern void timerInit (void);
extern void timer0Init (ushort);
extern void timer0Start(void);
extern void timer0Stop(void);
extern long timer0ReadTicks(void);
extern void timer1Init (ushort);
extern void timer1Start(void);
extern void timer1Stop(void);
extern void coldBlockTempInit(void);
extern void coldBlockTempSet(ushort myVal);
extern ushort coldBlockTempGet(void);
extern ushort isColdBlockTempOk(void);
extern void displayPrompt(uchar myPort);
extern short extPsaGet(void);

extern void sendStatusToGUI(char *);
extern void initEeprom(void);
extern int diagsEeprom(void);
extern void __swi(0) allInterruptsOn (void);
extern void __swi(1) allInterruptsOff (void);
extern void __swi(2) wDogKickSupr (void);

extern void vicDefInit(void);
extern void serialInit (void);
extern float basePlateTempGet(void);
//extern uchar basePlateTempDisplay(uchar myPort);
extern ulong errLogFlagGet(void);
extern void errLogDiscriptionGet(uchar myPort);
extern short isFaultActive(void);
void delayMs(ushort myDelay);
void wDogKick(void);
extern void storedFaultsClearAll(void);
extern ulong storedFaultsGet(void);
extern ulong prevErrLogFlagGet(void);
extern void prevErrLogDiscriptionGet(uchar myPort);
extern void tecHandlerProcess(ushort myMode);


short diagsDualPot(void);
void extQuadD2aInit(void);

//extA2d.c
short extA2dDiags(void);
void getAllExtA2dRegs(void);

uchar isA2dDone(void);
void a2dChnlStart(uchar);
ushort a2dChnlRead(void);

// i2c.c
void writeI2C2_2Bytes(uchar, uchar, uchar);
void writeI2C2_3Bytes(uchar, uchar, uchar, uchar);
void i2cWrite3Bytes(uchar chipAddr, uchar reg, uchar msb, uchar lsb);
void i2cWrite2Bytes(uchar chipAddr, uchar reg, uchar data);
void i2cRead2Bytes(uchar addr, uchar reg, uchar *myData, uchar mySize);
void readI2C2_BytesBlocking(uchar, uchar, uchar *, uchar );
void i2cInit(void);

// eeprom.h
void eepromInit(void);
char eepromRead(ushort myAddr, uchar* myData);
void eepromWrite(ushort , uchar );
char eepromReadbytes(uchar* , ushort );
void eepromWritebytes(uchar* , uchar , ushort );
char diagEeprom(void);

// extDualPotsDfb.c
void extDualPotsLsrDfbInit(void);
void extDualPotsLsrDfbSet(uchar myReg, uchar myValue);
uchar extDualPotsLsrDfbGet(uchar myReg);
void extDualPotsSoaSetCur(uchar myValue);
void extDualPotsDfbSetCur(uchar myValue);
uchar extDualPotsSoaGetCur(void);
uchar extDualPotsDfbGetCur(void);
void readAllExtPotsLsrDfbRegs(uchar myPort);
void potsLsrDfbWriteReg(ulong myULong1, ulong myULong2);
short diagsDualPotLsrDfb(void);

//extQuadD2a.c
void extQuadD2aInit(void);
void extQuadD2aSet(uchar myReg, ushort myLevel);
void extQuadD2aDfbGet(void);
void extQuadD2aSoaSetCur(ushort myLevel);
void extQuadD2aDfbSetCur(ushort myLevel);
void extQuadD2aSetHeater(ushort myLevel);
void extQuadD2aSoaSetLight(ushort myLevel);
ushort extQuadD2aSoaGetCur(void);
ushort extQuadD2aDfbGetCur(void);
ushort extQuadD2aGetHeater(void);
ushort extQuadD2aSoaGetLight(void);
void extQuadD2aLdacSetup(void);
void extQuadD2aRefSetup(void);
void extQuadD2aPwrUpDown(void);
void readAllExtD2aRegs(uchar myPort);
void writeExtD2aReg(ulong myULong1, ulong myULong2);
short diagsQuadD2a(void);

//extSinglePotLsr.c
void extSinglePotLsrSet(uchar myReg, uchar myValue);
void extSinglePotSetLtfb(uchar myValue);
uchar extSinglePotGetLtfb(void);
void readAllExtSingPotLsrRegs(uchar myPort);
void potsLsrSingleWriteReg(ulong myULong1, ulong myULong2);
short diagsSingPotLsr(void);

//extPsa.c
void extPsaSet(uchar myPort, uchar myVal);

