/*----------------------------------------------------------------------------
 *      Name:    basePlateTemp.c
 *      Purpose: Base Plate Temperature
 *
 *	BaseplateTemp - the temperature shall be sampled and if the value is out of 
 *	range, then additional action shall occur      
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include "system.h"
#include "type.h"
#include "serial.h"
#include "i2c.h"

static uchar basePlateTemp;			// range 0-55 degrees C

// get the base plate temperature by reading the A/D chnl, and calculating the temperature
float basePlateTempGet(void)
{	
	ushort myVal;
	float myFloat = 0;
	volatile short myTicks = 0;

  // get the Base Plate value
	a2dChnlStart(A2D_CHNL_BASE_PLATE);


 	timer0Init(MAX_TIMEOUT_A2D);
	timer0Start();

	while(isA2d0ConvDone == FALSE)
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			swError(__FILE__, __LINE__);
		 	break;
		}
	}

	timer0Stop();
	
	myVal = a2dChnlRead();

	// Base Plate Temp = 97.17 - (BASE PLATE TEMP, (ADC0.2) * 0.2182)
	myFloat = 97.17 - ( myVal * 0.2182);

	basePlateTemp = myFloat;

	return(basePlateTemp);
}

#if 0
// Output the baseplate temperature to the specified transport
uchar basePlateTempDisplay(uchar myPort)
{	
	ushort myVal;
	float myFloat = 0;
	char myBuf[20];
	volatile short myTicks = 0;

  // get the Base Plate value
	a2dChnlStart(A2D_CHNL_BASE_PLATE);

 	timer0Init(MAX_TIMEOUT_A2D);
	timer0Start();

	while(isA2d0ConvDone == FALSE)
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			swError(__FILE__, __LINE__);
		 	break;
		}
	}

	timer0Stop();
	
	myVal = a2dChnlRead();

	// Base Plate Temp = 97.17 - (BASE PLATE TEMP, (ADC0.2) * 0.2182)
	myFloat = 97.17 - ( myVal * 0.2182);

	sprintf(myBuf, "?BPT=%3.f", myFloat);
	putString(myPort, myBuf);

	basePlateTemp = myFloat;

	return(basePlateTemp);
}
#endif
