/*----------------------------------------------------------------------------
 *      Name:    coldBlockTemp.c
 *      Purpose: Cold Block temperature
 *
 *	ColdBlocktemp  the temperature shall be sampled and if the value is out of 
 *	range, then additional action shall occur.      
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/
#include "type.h"
#include "system.h"
#include "i2c.h"

#define DEFAULT_COLD_BLOCK_DAC_VAL	0x199

extern void d2aSet(ushort myVal);

static ushort coldBlockTemp;
short isTeCoolerHwOnBoard = TRUE;


void coldBlockTempInit(void)
{
	coldBlockTemp = 22;
	
	coldBlockTempSet(coldBlockTemp);
}



// accept a temp and convert to DAC value, set the DAC.
void coldBlockTempSet(ushort myTemp)
{
	ushort myCalc;

	coldBlockTemp = myTemp;

	myCalc = 34.1 * (myTemp - 10);
	
	d2aSet(myCalc);
}

// read the cold block temp
ushort coldBlockTempGet(void)
{
	ushort myVal;
	volatile short myTicks = 0;

	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!

	// get the Cold Block Value
	a2dChnlStart(A2D_CHNL_COLD_BLOCK);

 	timer0Init(MAX_TIMEOUT_A2D);
	timer0Start();

	while(isA2d0ConvDone == FALSE)
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			swError(__FILE__, __LINE__);
		 	break;
		}
	}

	timer0Stop();
	
	myVal = a2dChnlRead();

	coldBlockTemp = (myVal * 0.02933) + 10;

	return coldBlockTemp;
}

// determine if the cold block temperature is in a valid range
ushort isColdBlockTempOk(void)
{
	short myStatus = TRUE;

#if VORT_BOARD
	if(coldBlockTemp >= 15 && coldBlockTemp	<= 35)
		myStatus = TRUE;
	else
	{
		if(isTeCoolerHwOnBoard == TRUE)
				myStatus = FALSE;
		else
				myStatus = TRUE;		 	// if the TeCooler is not on the board, always return status that the TeCooler is fine.
	}	
#endif

	return myStatus;
}
