/*----------------------------------------------------------------------------
 *      Name:    errLog.h
 *      Purpose: Error Log
 *
 *	ErrLog  the error log literals
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#define MAX_ERRORS	0x20
#define MAX_ERR_RESPONSE 40

#define MAX_SW_ERRORS		 	10
#define MAX_SW_TRACES		 	20
#define MAX_SW_ERR_LEN		25

// Vortran Faults
// Do not modify this enum without modifying the "errDescription" structure
typedef enum
{
	ERR_LASER_EMISSION_ACTIVE = 0x0000,			// This is not an error!
	ERR_STANDBY 							= 0x0001,			// These are errors
	ERR_WARMUP 								= 0x0002,
	ERR_VALUE_OUT_OF_RANGE		= 0x0003,
	ERR_INVALID_COMMAND				= 0x0004,
	ERR_INTERLOCK_OPEN				= 0x0005,
	ERR_TEC_OFF								= 0x0006,
	ERR_DIODE_OVER_CURRENT		= 0x0007,
	ERR_DIODE_TEMP_FAULT			= 0x0008,
	ERR_BASE_PLATE_TEMP_FAULT	= 0x0009,
	ERR_BUFFER_OVERFLOW				= 0x000A,
	ERR_EEPROM_ERROR					= 0x000B,
	ERR_I2C_ERROR							= 0x000C,
	ERR_COMMAND_TIME_OUT			= 0x000D,
	ERR_WATCH_DOG_ERROR				= 0x000E,
	ERR_FATAL_ERROR						= 0x000F,
	ERR_END_OF_LIFE                 =0x0010,
	ERR_FREE_2,
	ERR_FREE_3,
	ERR_FREE_4,
	ERR_FREE_5
} errorFlags;

/* Cube Fault Codes
	0	System OK
	1	Bad Command
	2	Bad Data
	4	EEPROM Checksum Error
	8	Base Plate Temperature
	16	Laser Diode Temperature
	32	Interlock Fault
	64	I2C Error
	128	Value Out Of Range
	256	nc
	512	nc
	1024	Flash Reset
	2048	Over Current
	4096	Reset Clock
	8192	System Timeout
	16384	Buffer Overflow
	32768	Fatal Error
*/
// Do not modify this enum without modifying the "errDescription" structure
typedef enum
{
	ERR_SYSTEM_OK							= 0x0000,			// This is not an error!
	ERR_BAD_CMD 							= 0x0001,			// These are errors
	ERR_BAD_DATA							= 0x0002,
	ERR_EE_CHKSUM							= 0x0003,
	ERR_BASE_PLATE_TEMP				= 0x0004,
	ERR_LSR_DIODE_TEMP				= 0x0005,
	ERR_INTL_FAULT						= 0x0006,
	ERR_I2C										= 0x0007,
	ERR_VAL_OUT_OF_RANGE			= 0x0008,
	ERR_NC1										= 0x0009,
	ERR_NC2										= 0x000A,
	ERR_FLASH_RESET						= 0x000B,
	ERR_OVR_CUR								= 0x000C,
	ERR_RESET_CLK							= 0x000D,
	ERR_SYS_TIMEOUT						= 0x000E,
	ERR_BUF_OVR_FLOW					= 0x000F,
	ERR_FATAL									= 0x0010
} errorFlagsBrandx;


// bit encoded system hardware errors
typedef enum
{
	NO_SYS_HW_ERRS 	= 	0x0000,
	HW_ERR_EEPROM 	= 	0x0001,
	HW_ERR_A2D 		= 	0x0002,
	HW_ERR_QUAD_D2A = 	0x0004,
	HW_ERR_POT_LSR 	= 	0x0008,
	HW_ERR_POT_MOD 	= 	0x0010,
	HW_ERR_PSA		= 	0x0020,
	HW_ERR_A2D_DFB  =	0x0040,
	HW_ERR_SINGLE_POT_LSR = 0x0080
} SYS_HW_ERRS;

extern SYS_HW_ERRS sysHwStatus;		// bit encoded status of hardware devices
extern void clearSysHwErr(void);
extern SYS_HW_ERRS getSysHwErr(void);
extern void setSysHwErr(SYS_HW_ERRS);


// prototypes
extern void errLogInit(void);
extern void errLogFlagClearAll(void);
extern void errLogFlagClear(ulong myError);
extern void errLogFlagSet(ulong myError);
extern short isErrLogStatGrtr4(void);
extern short isErrLogStatGrtr16(void);
extern short isErrLogStatGrtr32(void);
extern void saveSwErrChkSum(void);
extern void saveSwErrDefaults(void);



