
/*----------------------------------------------------------------------------
 *      Name:    extSinglePotLsr.c
 *      Purpose: External Single Pot
 *
 * Purpose: SOA Light Feedback Channel Calibration
 *
 *The AD5259 provides a compact, nonvolatile LFCSP-10 (3 mm  3 mm) or MSOP-10 
 (3 mm  4.9 mm) packaged solution for 256-position adjustment applications. 
 These devices perform the same electronic adjustment function as mechanical 
 potentiometers1 or variable resistors, but with enhanced resolution and solid-state 
 reliability. The wiper settings are controllable through an I2C-compatible digital 
 interface that is also used to read back the wiper register and EEPROM content. 
 Resistor tolerance is also stored within EEPROM, providing an end-to-end tolerance 
 accuracy of 0.1%.       
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "system.h"
#include "type.h"
#include "i2c.h"
#include "serial.h"
#include "extSinglePot.h"
#include "errLog.h"



void extSinglePotLsrInit(void)
{
	// read a register in the POT to verify that it is working
#if 0
	extSinglePotGetLtfb();
	delayMs(5);

	if(isrI2cErr == TRUE)		// did the A/D fail?
	{
		setSysHwErr(HW_ERR_SINGLE_POT_LSR);	// yes
		isrI2cErr = FALSE;
	}	
#endif
}

void extSinglePotLsrSet(uchar myReg, uchar myValue)
{
	writeI2C2_2Bytes(I2C_ADDR_SING_POT_LSR, myReg, myValue);
	writeI2C2_Byte(I2C_ADDR_SING_POT_LSR, myReg | S_POT_STORE_CMD);
}


void extSinglePotSetLtfb(uchar myValue)
{
	extSinglePotLsrSet(SINGLE_POT_CH1, myValue);
}

uchar extSinglePotGetLtfb(void)
{
	uchar myVal = 0;

	readI2C2_BytesBlocking(I2C_ADDR_SING_POT_LSR, SINGLE_POT_CH1, (uchar *) &myVal, 1);
	delayMs(2);

	return(myVal);
}

void readAllExtSingPotLsrRegs(uchar myPort)
{
	uchar myVal;
	char myBuf[20];

	readI2C2_BytesBlocking(I2C_ADDR_SING_POT_LSR, SINGLE_POT_CH1, (uchar *) &myVal, 1);
	delayMs(2);
	sprintf(myBuf, "RDAC1: %d\n", myVal);
	putString(myPort, myBuf);
}

void potsLsrSingleWriteReg(ulong myULong1, ulong myULong2)
{
	if(myULong1 == 95)
 	{
		extSinglePotSetLtfb(myULong2);
	}
}

short diagsSingPotLsr(void)
{
	short myStatus = FAIL;

	return(myStatus);
}
