/*----------------------------------------------------------------------------
 * Name:    demo.c
 * Purpose: USB HID Demo
 * Version: V1.20
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2008 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <LPC214X.H>                        /* LPC214x definitions */

#include "type.h"
#include "system.h"
#include "queue.h"

#include "usb.h"
#include "usbcfg.h"
#include "usbhw.h"

#include "demo.h"
#include <stdio.h>
#include <string.h>
#include "hidReport.h"

char myUsbOutBuf[64] = {0};
char myUsbInBuf[64] = {0};
char myUsbInTmpBuf[64] = {0};
char myUsbInTmpBuf2[64] = {0};

U8 InReport[USB_MAX_PACKET0];                                /* HID Input Report    */
U8 OutReport[USB_MAX_PACKET0];                               /* HID Out Report      */

U8 tempReport[USB_MAX_PACKET0];
U8 tempInReport[USB_MAX_PACKET0];

uchar wrQSize = 0;
volatile BYTE UsbInPending = FALSE;
U8 RequestType = 0x00;
U8 tempMessage[64];

/*------------------------------------------------------------------------------
  Get HID Input Report -> InReport
 *------------------------------------------------------------------------------*/

void GetInReport (void)
{
	
}

/*------------------------------------------------------------------------------
  Set HID Output Report <- OutReport
 *------------------------------------------------------------------------------*/
 void SetOutReport (void)
 {
 	int idx;

  	memset(InReport,0x00,64); //Delete stale previous request
  	memset(tempReport,0x00,64);
	memset(tempInReport,0x00,64);
	memcpy(tempReport,OutReport,63);
		   
  	//Isolate request type from tempReport packet
  	RequestType = tempReport[0];

  	//Filter out request type from packet and load it into response to PC along with a parameter or message
  	switch(RequestType)
  	{
  		case (U8)0xA0://SET_COMMAND_QUERY:
			//Send OUT buffer contents to parser
			for(idx=0;idx<64;idx++)
			{
				if(tempReport[idx]==0xFF)
					tempReport[idx]=0x00;
			}
			memcpy(tempMessage,&tempReport[1],63);
			writeQ(USB_Q, IN_Q, (uchar *)tempMessage, strlen((const char*) tempMessage));
			//memset(myUsbOutBuf, 0, sizeof(tempReport));
			break;
    	case (U8)0xA1://GET_RESPONSE_STATUS:
			//Return GET_RESPONSE_STATUS request type and its current value
			tempInReport[0] = (U8)GET_RESPONSE_STATUS;
			tempInReport[1] = UsbInPending;
			for(idx=2;idx<64;idx++)
			{
		 		tempInReport[idx]=0xFF; //Fill with 0xFF to reduce EMI
			}
			memcpy(InReport,tempInReport,64);
			break;
    	case (U8)0xA2://GET_RESPONSE:
			//Return GET_RESPONSE request type and the response string
			//Load response into buffer
			tempInReport[0] = (U8)GET_RESPONSE;
			memcpy(&tempInReport[1],myUsbInTmpBuf,63);
			memset(myUsbInTmpBuf,64,0x00);
			memcpy(InReport,tempInReport,64);
			break;
    	case (U8)0xA3://SET_RESPONSE_RECEIVED:
			//Clear buffer, clear UsbInPending condition, send nothing to host
			memset(&tempInReport[1],0x00,63);
			UsbInPending = FALSE;
			break;
		default:
			//Unsupported!!
			break;
  	}




 }

