/************************************************************/
/* PROJECT NAME: I2C 	                                    */
/* Project:      LPC2100 Training course                    */
/* Engineer:     T Martin      tmartin@hitex.co.uk          */
/* Filename:     MAIN.C                                     */
/* Language:     C                      	                */
/* Compiler:     Keil ARM	V1.3		                    */
/* Assembler:    					                        */
/*                                                          */
/************************************************************/
/* COPYRIGHT: Hitex UK Ltd 		2004						*/
/* LICENSE:   THIS VERSION CREATED FOR FREE DISTRIBUTION	*/
/************************************************************/
/* Function:                                                */
/*                                                          */
/* Interrupt driven I2C example for LPC2106                 */
/*															*/
/* The Transfer byte routine starts a single byte transfer  */
/*  on the I2C bus. The development board has two port 		*/
/* expanders one at address 0x42 with a 7 segment LCD a  	*/
/* second at 0x41 with a four switch header. The code reads	*/
/* the switches and writes the pattern to the LED's			*/
/*															*/	
/************************************************************/

//#include <LPC210x.H> 
#include <LPC214X.H>                        /* LPC214x definitions */
#include <stdarg.h>
#include "system.h"
#include "i2c.h"
#include "type.h"

#ifdef SW_DEBUG
uchar I2cStat[20];			// debug info
#endif

#define MAX_TIMEOUT_I2C	3

void I2CISR (void)	__irq ;																			//I2C interrupt routine
void I2CTransferByte(unsigned int I2CAddr,unsigned char MemAddr,unsigned char count,...);			//Background call to start master send and receive byte transfers

short isrI2cErr = FALSE;

static uchar 	*I2CData;
static char 	I2Counter;
static uchar	I2CAddress;
static uchar	MemAddress;
static uchar	lock;		//Define Function prototypes and Global variables
static ushort nakExtA2D = FALSE;


// Inititalize I2C 0 for communication with
// eeprom, D/A, A/D, POTs
void i2CInit(void)
{
	uchar myTmp;

#ifdef SW_DEBUG
	uchar myCnt;
#endif

#ifdef SW_DEBUG	
	for(myCnt = 0; myCnt < 20; myCnt++)
		I2cStat[myCnt] = 0;	
#endif	

	// re-initialize the eeprom
	// 3.16 Bus clear
	// In the unlikely event where the clock (SCL) is stuck LOW, the preferential procedure is to
	// reset the bus using the HW reset signal if your I2C devices have HW reset inputs. If the
	// I2C devices do not have HW reset inputs, cycle power to the devices to activate the
	// mandatory internal Power-On Reset (POR) circuit.
	// If the data line (SDA) is stuck LOW, the master should send 9 clock pulses. The device
	// that held the bus LOW should release it sometime within those 9 clocks. If not, then use
	// the HW reset or cycle power to clear the bus.
	for(myTmp = 0; myTmp < 9; myTmp++)
	{
	 	IO0SET = PIN_SCL0;
		delayMs(1);
	 	IO0CLR = PIN_SCL0;
		delayMs(1);
	}							
	
	// I2C Port 0, not port 0.
	VICVectCntl1 = 0x20 | 9;  						//select a priority slot for a given interrupt	I2C0
	VICVectAddr1 = (unsigned)I2CISR;			//pass the address of the IRQ into the VIC slot
	VICIntEnable = 0x00000200;						//enable interrupt

	PINSEL0  	 |= 0x00000050; 				//Switch GPIO to I2C pins for I2C 1
	
	//I2C0SCLH 		 = 0x13;						//Set bit rate 12Mhz/(VPBDIV+SCLH+SCLL) = 12MHz/ (17 + 17) = ~300Khz
	//I2C0SCLL 		 = 0x13;

	I2C0SCLH 		 = 0x50;						//Set bit rate 12Mhz/(VPBDIV+SCLH+SCLL) = 12MHz/ (0x50 + 0x50) = ~70Khz
	I2C0SCLL 		 = 0x50;
}

// Perform an I2C transfer of more then 1 byte to the specified external I2C device based on "device address"
void I2CTransferByte(unsigned int I2CAddr,unsigned char MemAddr,unsigned char count,...)
{
	va_list ap;
	va_start(ap,count);
	
	I2CAddress 	= I2CAddr;						//Place address and data in Globals to be used by the interrupt
	
	if(count >0)
	{
		I2CData  	= va_arg(ap,unsigned char *);
	}

	I2Counter	= count;
	MemAddress	= MemAddr;

	I2C0CONCLR 	= 0x000000FF;						//Clear all I2C settings
	I2C0CONSET 	= 0x00000040; 					//Enable the I2C interface
	I2C0CONSET 	= 0x00000020; 					//Start condition
	va_end(ap);
}

// Perform an I2C transfer of 1 byte "device address" to the specified external I2C device 
void I2CTransferOneByte(unsigned int I2CAddr)
{	
	I2CAddress 	= I2CAddr;						//Place address and data in Globals to be used by the interrupt
	
	I2Counter	= 0;
	MemAddress	= 0xFF;		 						// FRG Fix Later!!!

	I2C0CONCLR 	= 0x000000FF;					//Clear all I2C settings
	I2C0CONSET 	= 0x00000040; 				//Enable the I2C interface
	I2C0CONSET 	= 0x00000020; 				//Start condition
}


// I2C Read Bytes
void I2CReadBytes(unsigned int I2CAddr,unsigned char count,...)
{
	va_list ap;
	va_start(ap,count);
	
	I2CAddress 	= I2CAddr;						//Place address and data in Globals to be used by the interrupt
	
	if(count >0)
	{
		I2CData  	= va_arg(ap,unsigned char *);
	}

	I2Counter	= count;

	I2C0CONCLR 	= 0x000000FF;					//Clear all I2C settings
	I2C0CONSET 	= 0x00000040; 					//Enable the I2C interface
	I2C0CONSET 	= 0x00000020; 					//Start condition
	va_end(ap);
}


void I2CISR (void)	__irq						//I2C interrupt routine
{
	static uchar nakAttempts = 0;


#ifdef SW_DEBUG
	static uchar myCnt = 0;

	wasHere ==0x110;

	if(myCnt < 20)
		I2cStat[myCnt++] = I2C0STAT;
#endif

	switch (I2C0STAT)								//Read result code and switch to next action
	{
		// Start and Send byte conditions
		
		case ( 0x08):								//Start bit
			I2C0CONCLR 	= 0x20;							//Clear start bit
			I2C0DAT   	= I2CAddress; 					//Send address and write bit
			nakAttempts = 0;
			break;
		
		case (0x18):								//Slave address+W, ACK
			I2C0DAT   	= MemAddress;					//Write Memory start address to tx register
			break;
		
		case (0x20):								//Slave address +W, Not ACK
			I2C0DAT   	= I2CAddress; 					//Resend address and write bi
			break;
		
		case (0x28):
			//if(I2Counter-->0)								//Data sent, Ack
			if(I2Counter-- != 0)							//Data sent, Ack
			{
				I2C0DAT   	= *I2CData;						//Write data to tx register
				I2CData++;
			}
			else
			{
				I2C0CONSET 	= 0x10;							//Stop condition
				lock = 0;                       			//Signal end of I2C activity
			}
			break;
		
		case (0x30)	:								//Data sent, NOT Ack
			if(nakAttempts++ < 5)
				I2C0DAT   	= *I2CData;						//Write data to tx register
			else
			{
				nakAttempts = 0;
				I2C0CONSET 	= 0x10;							//Stop condition
				lock 		= 0;                    		//Signal end of I2C activity
				
				isrI2cErr = TRUE;
			}
		break;
		
		
		//Receive byte conditions
		
		case (0x40) :								//Slave Address +R, ACK
			I2C0CONSET 	= 0x04;							//Enable ACK for data byte
			break;
		
		case (0x48) :								//Slave Address +R, Not Ack
			I2C0CONSET 	= 0x20;							//Resend Start condition
			break;
		
		case (0x50) :								//Data Received, ACK 
			if(--I2Counter > 0)
			{
				*I2CData 	= I2C0DAT;
				I2CData++;

				if((I2Counter == 1) && (nakExtA2D == TRUE))
				{
					I2C0CONCLR 	= 0x04;							//Disable ACK for data byte
				}
			}
			else
			{
				*I2CData 	= I2C0DAT; 						// no more bytes to receive! Get the current byte, and send a stop condition.
				I2CData++;

				I2C0CONSET 	= 0x10;							//Stop condition
				lock 		= 0;                    		//Signal end of I2C activity							
			}

			break;
		
		case (0x58):								//Data Received, Not Ack
			if((I2Counter == 1) && (nakExtA2D == TRUE))	// special condition for external A2D: NAKs last byte received
			{
				*I2CData 	= I2C0DAT; 						// no more bytes to receive! Get the current byte, and send a stop condition.
				I2CData++;
				nakExtA2D	= FALSE;
				I2C0CONSET 	= 0x10;							//Stop condition
				lock 		= 0;                    //Signal end of I2C activity
			}
			else
				I2C0CONSET 	= 0x20;							// Resend Start condition

			break;
		
		default :
			break;
	
	}

	I2C0CONCLR 	= 0x08;							//Clear I2C interrupt flag
	VICVectAddr = 0x00000000;					//Clear interrupt in 

}


// Write 1 bytes from an I2C device, based on the register address 
void writeI2C2_2Bytes(uchar chipAddr, uchar reg, uchar data)
{
	static uchar myMsg[1];
	volatile short myTicks = 0;

	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			lock = 0;
			swError(__FILE__, __LINE__);
		 	break;
		}

	timer0Stop();

	lock 		= 1;                   			//Set I2C bus as active		
	myMsg[0] = data;
	
	I2CTransferByte(chipAddr | I2C_WRITE, reg, 1, myMsg);			//write data to the I2C Memory
}

// Write 1 bytes to an I2C device, based on the register address 
void writeI2C2_Byte(uchar chipAddr, uchar data)
{
	static uchar myMsg[1] = {0};
	volatile short myTicks = 0;

	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}

	timer0Stop();

	lock 		= 1;                   			//Set I2C bus as active		
	
	myMsg[0] = data;
	
	I2CTransferByte(chipAddr | I2C_WRITE, data, 0, myMsg);			//write data to the I2C Memory
}




// Write 2 bytes from an I2C device, based on the register address 
void writeI2C2_3Bytes(uchar chipAddr, uchar myMsb, uchar myLsb, uchar myData)
{
	static uchar myMsg[2];
	volatile short myTicks = 0;

	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			lock = 0;
			swError(__FILE__, __LINE__);
		 	break;
		}

 	timer0Stop();

	lock 		= 1;                   			//Set I2C bus as active			
	
		myMsg[0] = myLsb;
		myMsg[1] = myData;
		I2CTransferByte(chipAddr | I2C_WRITE, myMsb, 2, myMsg);			//write data to the I2C Memory
}


// Read a byte from a specified 16 bit address of an I2C device
void readI2C2_16BitAddr_BytesBlocking(uchar addr, ushort reg, uchar *myData, uchar mySize)
{
	static uchar myLsb[1];
	volatile short myTicks = 0;

	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}

 	timer0Stop();

	lock 		= 1;                   			//Set I2C bus as active
	
	myLsb[0] = reg & 0xFF;
	I2CTransferByte(addr | I2C_WRITE, (reg & 0xFF00) >> 8, 1, myLsb);			// select the port which shall be read
	
	
	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}
	}

 	timer0Stop();

	lock 		= 1;  
	
	I2CTransferByte(addr | I2C_READ, reg, mySize, myData);		//read back data, blocking call for I2C

}


// Read 2 bytes from an I2C device, based on the register address
void readI2C2_BytesBlocking(uchar addr, uchar reg, uchar *myData, uchar mySize)
{
	int myStatus = PASS;
	volatile short myTicks = 0;

	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}

 	timer0Stop();

	if(myStatus == PASS)
	{
		lock 		= 1;                   			//Set I2C bus as active
	
	
		I2CTransferByte(addr | I2C_WRITE, reg, 0, 0);			// select the port which shall be read
	}
	else
		return;


	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			//myStatus = FAIL;
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}
	}

 	timer0Stop();

	if(myStatus == PASS)
	{
		lock 		= 1;  
	
		I2CTransferByte(addr | I2C_READ, reg, mySize, myData);		//read back data, blocking call for I2C
	}
	else
		return;
}

// Read 2 bytes from an I2C device, NON REGISTER based (Only the address is used)
void readI2C2_BytesBlockingNR(uchar addr, uchar *myData, uchar mySize)
{
	int myStatus = PASS;
	volatile short myTicks = 0;

	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}

 	timer0Stop();

	if(myStatus == PASS)
	{
		lock 		= 1;                   			//Set I2C bus as active
	
	
		I2CTransferOneByte(addr | I2C_WRITE);			// select the port which shall be read
	}
	else
		return;


	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			//myStatus = FAIL;
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}

 	timer0Stop();

	if(myStatus == PASS)
	{
		lock 		= 1;  
	
		// FIX LATER!!! Reg is not used!!    I2CTransferByte(addr | I2C_READ, reg, mySize, myData);		//read back data, blocking call for I2C
	}
	else
		return;
}

// Read 3 bytes from an I2C device, the register address is not used
void readI2C2_Dual_D2a_BytesBlocking(uchar addr, uchar *myData, uchar mySize)
{
	int myStatus = PASS;
	uchar reg = 0; //not used
	volatile short myTicks = 0;

	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			//myStatus = FAIL;
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}

 	timer0Stop();

	if(myStatus == PASS)
	{
		lock 		= 1;
		
		nakExtA2D = TRUE;  
	
		I2CTransferByte(addr | I2C_READ, reg, mySize, myData);		//read back data, blocking call for I2C
	}
	else
		return;
}

// Read 6 bytes from the A2D I2C device, the register address is not used, the previous write sets up the read.
// Note: send a NAK for the last byte received to let the A2D I2C device know that no additional samples are needed
void readI2C2_Dual_A2d_BytesBlocking(uchar addr, uchar *myData, uchar mySize)
{
	int myStatus = PASS;
	uchar reg = 0; //not used
	volatile short myTicks = 0;

	timer0Init(MAX_TIMEOUT_I2C);
	timer0Start();

	// blocking, until previous command is complete!!
	while(lock == 1)							//Wait for interrupt to signal end of I2C activity
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			//myStatus = FAIL;
			lock = 0;
		 	swError(__FILE__, __LINE__);
			break;
		}

 	timer0Stop();

	if(myStatus == PASS)
	{
		lock 		= 1; 
		
		nakExtA2D = TRUE; 
	
		I2CTransferByte(addr | I2C_READ, reg, mySize, myData);		//read back data, blocking call for I2C
	}
	else
		return;
}

