/*----------------------------------------------------------------------------
 *      Name:    parser.h
 *      Purpose: interpret commands
 *
 *	Parser  shall parse commands from serial and or USB. The command shall
 *	perform an operation and or return status.
 *
 *	High Priority Commands: Laser Current, Laser Power, and Laser On/Off
 *	Synchronization numbers
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/


#define MAX_CMD_RSP_SIZE 		80
#define CR_KEY							0x0D					// Carriage return
#define NL_KEY							0x0A					// New Line
#define BS_KEY							0x08					// Backspace

#define LSR_POWER_CONTROL			0
#define LSR_CURRENT_CONTROL 	1

//PUL=1 at powerup or user initiated PUL=1
#define PUL_AT_POWERUP 1
#define PUL_USER_INIT 0



#define MAX_PROTO_CMD_SIZE	 10
typedef struct {
	char cmd[MAX_PROTO_CMD_SIZE];
	uchar maxArgs;
	ushort (*cmdHandler)() ;
} PROTOCOL_CMDS;

void pulseModeOn(void);

ushort lpHandler(void);
ushort cHandler(void);
ushort cfcHandler(void);
ushort dfblcHandler(void);
void dfblcHandlerProcess(ushort myMode);
ushort rfcHandler(void);
ushort lcHandler(void);
ushort leHandler(void);
ushort maxpHandler(void);
ushort epcHandler(void);
ushort delayHandler(void);
ushort vortHandler(void);
ushort vobtsHandler(void);
ushort tecHandler(void);
ushort echoHandler(void);
ushort promptHandler(void);
ushort protoHandler(void);
ushort opHandler(void);
ushort ppHandler(void);
ushort pulHandler(void);

ushort monCdrhHandler(void);	// enable/disable monitor laser conditions: CDRH
ushort monStdbHandler(void);	// enable/disable monitor laser conditions: Stand By
ushort monIntlHandler(void);	// enable/disable monitor laser conditions: Interlock open
ushort monDiCuHandler(void);	// enable/disable monitor laser conditions: Diode Current
ushort monDTmpHandler(void);	// enable/disable monitor laser conditions: Diode Temp
ushort monBPltHandler(void);	// enable/disable monitor laser conditions: Base Plate Temp
ushort burnHandler(void);			// eeprom set to defaults
ushort tecModeHandler(void);
ushort psaHandler(void);
ushort swErrHandler(void);		// display sw error handle
ushort wDogOffHandler(void);
ushort rWDogHandler(void);

ushort rBptHandler(void);
ushort rLcHandler(void);
ushort rLcsHandler(void);
ushort rObtsHandler(void);
ushort rObtHandler(void);
ushort rDelayHandler(void);
ushort rDfblcHandler(void);
ushort rDfblcsHandler(void);
ushort rEchoHandler(void);
ushort rEpcHandler(void);
ushort rFcHandler(void);
ushort rFdHandler(void);
ushort rFfHandler(void);
ushort rHelpHandler(void);
ushort rHlcHandler(void);
ushort rLeHandler(void);
ushort rLhHandler(void);
ushort rLiHandler(void);
ushort rFpHandler(void);
ushort rLpsHandler(void);
ushort rLpHandler(void);
ushort rLsHandler(void);
ushort rFvHandler(void);
ushort rCHandler(void);
ushort rCcHandler(void);
ushort rMaxpHandler(void);
ushort rRpHandler(void);
ushort rLwHandler(void);
ushort rIlHandler(void);
ushort rSsHandler(void);
ushort rStaHandler(void);
ushort rTecHandler(void);
ushort rPsaHandler(void);
ushort rOpHandler(void);
ushort rPpHandler(void);
ushort rPulHandler(void);
ushort rOpStat1Handler(void);
ushort rOpStat2Handler(void);
ushort rOpStat3Handler(void);
ushort rOpStat4Handler(void);
ushort rOpStat5Handler(void);

ushort rMonCdrhHandler(void);	// enable/disable monitor laser conditions: CDRH
ushort rMonStdbHandler(void);	// enable/disable monitor laser conditions: Stand By
ushort rMonIntlHandler(void);	// enable/disable monitor laser conditions: Interlock open
ushort rMonDiCuHandler(void);	// enable/disable monitor laser conditions: Diode Current
ushort rMonDTmpHandler(void);	// enable/disable monitor laser conditions: Diode Temp
ushort rMonBPltHandler(void);	// enable/disable monitor laser conditions: Base Plate Temp

ushort calDacHandler(void);
ushort calDacDfbHandler(void);
ushort calCalVerHandler(void);
ushort calDfblcDacCntHandler(void);
ushort calDfblcsdHandler(void);
ushort calDfblcspHandler(void);
ushort calMaxPwrDfbHandler(void);
ushort calMaxPwrLsrHandler(void);
ushort calHtrSetDacHandler(void);
ushort calInitLsrCurHandler(void);
ushort calInitDfbCurHandler(void);
ushort calNomLsrPwrHandler(void);
ushort calMaxLsrCurHandler(void);
ushort calMaxLsrCurDfbHandler(void);
ushort calMaxHtrCurHandler(void);
ushort calMaxOverCurHandler(void);
ushort calNomWaveLenHandler(void);
ushort calMeasWaveLenHandler(void);
ushort calLsrPwrSetDacHandler(void);
ushort calLsrPwrSetPotHandler(void);
ushort calLsrCurSetDacHandler(void);
ushort calLsrCurSetPotHandler(void);
ushort calLsrPartNumHandler(void);
ushort calLsrSerialNumHandler(void);
ushort calLsrBeamShapeHandler(void);
ushort calObtDeviationHandler(void);
ushort calLcScaleFactorHandler(void);
ushort calEpcScaleFactorHandler(void);
ushort calLpoaHandler(void);
ushort calEOLHandler(void);
ushort calEOL_FaultHandler(void);
ushort calNVPULHandler(void);
ushort calEpcScaleFactor2Handler(void);
ushort calModeHandler(void);
ushort calRfInjHandler(void);
ushort calHtrCntHandler(void);



// Calibration command responses
ushort rcalDacHandler(void);
ushort rcalDacDfbHandler(void);
ushort rcalCalVersionHandler(void);
ushort rcalDfblcDacCntHandler(void);
ushort rcalDfblcspHandler(void);
ushort rcalHelpHandler(void);
ushort rcalMaxPwrDfbHandler(void);
ushort rcalMaxPwrLsrHandler(void);
ushort rcalInitLsrCurHandler(void);
ushort rcalInitDfbCurHandler(void);
ushort rcalNomLsrPwrHandler(void);
ushort rcalMaxLsrCurDfbHandler(void);
ushort rcalMaxLsrCurHandler(void);
ushort rcalMaxHtrCurHandler(void);
ushort rcalMaxOverCurHandler(void);
ushort rcalNomWaveLenHandler(void);
ushort rcalMeasWaveLenHandler(void);
ushort rcalLsrPwrSetPotHandler(void);
ushort rcalLsrCurSetPotHandler(void);
ushort rcalLsrPartNumHandler(void);
ushort rcalLsrSerialNumHandler(void);
ushort rcalLsrBeamShapeHandler(void);
ushort rcalObtDeviationHandler(void);
ushort rcalLcScaleFactorHandler(void);
ushort rcalEpcScaleFactorHandler(void);
ushort rcalLpoaHandler(void);
ushort rcalEOLHandler(void);
ushort rcalEOL_FaultHandler(void);
ushort rcalNVPULHandler(void);
ushort rcalEpcScaleFactor2Handler(void);
ushort rcalModeHandler(void);
ushort rcalRfInjHandler(void);
ushort rcalHtrCntHandler(void);

// Debug Commands
ushort rHandler(void);
ushort wHandler(void);
ushort gpHandler(void);
ushort dSysCfgHandler(void);
ushort forceVerHandler(void);
ushort cpSwErrHandler(void);
ushort dpSwErrHandler(void);
ushort cpHandler(void);
ushort spHandler(void);
ushort lhHandler(void);
ushort hdHandler(void);



