/*----------------------------------------------------------------------------
 *      Name:    serial.c
 *      Purpose: Serial Driver
 *      
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

//#include <stdio.h>
#include <LPC214x.H>

#include "system.h"
#include "serial.h"
#include "queue.h"
#include "sysCfg.h"
#include "type.h"
#include "parser.h"
#include "string.h"

#define CR     0x0D


//UART Interrupt Identification Register
#define RX_LINE_STATUS 0x06
#define RX_DATA_AVAIL 0x04
#define CHAR_TIMEOUT 0x0C
#define THRE_INTERRUPT 0x02
#define INT_PENDING 0x01    //If this bit is 0 then an interrupt is pending

 
//int putchar1 (int ch);
//int getchar1 (void);
#define MAX_SERIAL_BUF_SIZE	0x40

#define MAX_TIMEOUT_USB				5
#define MAX_USB_TIMEOUT_IN_TICKS	800 //was 200

// prototypes
void putString(uchar myPort, char *);
//int putcharComm1 (int ch); 			  // Write character to Serial Port  

//uchar serialRxBuf[MAX_SERIAL_BUF_SIZE];
//uchar serialTxBuf[MAX_SERIAL_BUF_SIZE];

char comm0RxBuf[MAX_CMD_SIZE];
char comm0RxInPos = 0;
char comm0RxOutPos = 0;

//uchar* serialTxInPtr;
//uchar* serialTxOutPtr;
//uchar* serialRxInPtr;
//uchar* serialRxOutPtr;

long rd1InBuf, wr1InBuf;
long rd1OutBuf, wr1OutBuf;
uchar charsToProcessRx0 = 0;
uchar myCount = 0;

char rx0Buf[MAX_CMD_SIZE];
char tx0Buf[MAX_CMD_SIZE];
char rx1Buf[MAX_CMD_SIZE];
char tx1Buf[MAX_CMD_SIZE];

uchar mySerial0In[MAX_SERIAL_BUF_SIZE];
uchar mySerial0Pos = 0;
uchar mySerial1In[MAX_SERIAL_BUF_SIZE];
uchar mySerial1Pos = 0;

long debugCount=0;


// implementation of putchar (also used by printf function to output data)    
int sendchar (uchar myPort, int myChar)                   // Write character to Serial Port    
{
	if(myPort == COMM_0)
	{
	  if (myChar == '\n')  {
	    while (!(U0LSR & 0x20));
	    U0THR = CR;                          // output CR 
	  }

	  while (!(U0LSR & 0x20))
			;

	  return (U0THR = myChar);
	}
	else
	{
	  if (myChar == '\n')  {
	    while (!(U1LSR & 0x20));
	    U1THR = CR;                          // output CR 
	  }

	  while (!(U1LSR & 0x20))
			;

	  return (U1THR = myChar);
	}
}



void serialInit(void)
{
	while(mySerial0Pos < MAX_SERIAL_BUF_SIZE)
		mySerial0In[mySerial0Pos++] = 0;
	
	mySerial0Pos = 0;

	VPBDIV = 0x01;									//Divide Pclk by one = 60MHz

	//////////////
	// Comm 0
	//////////////
  PINSEL0 	|= 0x00000005;       	//Enable RxD0 and TxD0
  U0LCR 	= 0x83;         				// 8 bits, no Parity, 1 Stop bit
	//U0DLM = 1;             					// 9600 Baud Rate @ 60MHz VPB Clock 
	//U0DLL = 135;           					// 9600 Baud Rate @ 60MHz VPB Clock
	U0DLM = 0;             					// 19200 Baud Rate @ 60MHz VPB Clock 
	U0DLL = 195;           					// 19200 Baud Rate @ 60MHz VPB Clock
	//U0DLM = 0;             					// 38400 Baud Rate @ 60MHz VPB Clock 
	//U0DLL = 98;           					// 38400 Baud Rate @ 60MHz VPB Clock
	//U0DLM = 0;             					// 56800 Baud Rate @ 60MHz VPB Clock 
	//U0DLL = 66;           					// 56800 Baud Rate @ 60MHz VPB Clock   
	//U0DLM = 0;             					// 115200 Baud Rate @ 60MHz VPB Clock 
	//U0DLL = 33;           					// 115200 Baud Rate @ 60MHz VPB Clock   
	U0LCR 	= 0x03;        					// DLAB = 0 

	// Enable Serial Comm 0 Interrupts
//#if 0
	U0IER = 0x05; 				//disabled THRE int, enabled rx line interrupt enable
	U0FCR = 0x41;					//Enable FIFO with 4 Byte Trigger
	VICVectAddr6 = (unsigned long)uart0_isr;   
	VICVectCntl6 = 0x20 | 6;                  /* Uart0 Interrupt -> IRQ Slot 9 */
	VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
//#endif

#if ACTIVATE_COMM1
	//////////////
	// Comm 1
	//////////////
  PINSEL0 	|= 0x00050000;       	//Enable RxD1 and TxD1
  U1LCR 	= 0x83;         				// 8 bits, no Parity, 1 Stop bit
	//U1DLM = 1;             					// 9600 Baud Rate @ 60MHz VPB Clock 
	//U1DLL = 135;           					// 9600 Baud Rate @ 60MHz VPB Clock
	U1DLM = 0;             					// 19200 Baud Rate @ 60MHz VPB Clock 
	U1DLL = 195;           					// 19200 Baud Rate @ 60MHz VPB Clock
	//U1DLM = 0;             					// 38400 Baud Rate @ 60MHz VPB Clock 
	//U1DLL = 98;           					// 38400 Baud Rate @ 60MHz VPB Clock
	//U1DLM = 0;             					// 56800 Baud Rate @ 60MHz VPB Clock 
	//U1DLL = 66;           					// 56800 Baud Rate @ 60MHz VPB Clock   
	//U1DLM = 0;             					// 115200 Baud Rate @ 60MHz VPB Clock 
	//U1DLL = 33;           					// 115200 Baud Rate @ 60MHz VPB Clock   
	U1LCR 	= 0x03;        					// DLAB = 0 


	// Enable Serial Comm 1 Interrupts
	U1IER = 0x05; 				//disabled THRE int, enabled rx line interrupt enable
	U1FCR = 0x41;					//Enable FIFO with 4 Byte Trigger

	VICVectAddr5 = (unsigned long)uart1_isr;   
	VICVectCntl5 = 0x20 | 7;                  /* Uart1 Interrupt -> IRQ Slot 5 */

	VICIntEnable = 1 << 7;       /* Enable UART1 Interrupt */
#endif
}


int putchar (uchar myPort, int myChar) 			  /* Write character to Serial Port    */
{
	if(myPort == COMM_0)
	{
	  if (myChar == '\n')  
	  {
		VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt */
	    while (!(U0LSR & 0x20));
	    U0THR = CR;              	   /* output CR */
  		VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
	  }

		VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt */
	  while (!(U0LSR & 0x20));

  	VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
			
	  return (U0THR = myChar);
	}
	else
	{                  		
	  if (myChar == '\n')  
		{
	    while (!(U1LSR & 0x20));
	    U1THR = CR;              	   /* output CR */
	  }

	  while (!(U1LSR & 0x20));

	  return (U1THR = myChar);
	}
}

void putString (uchar myPort, char* myBuf) 			  /* Write characters to Serial Port, or USB port    */
{
	volatile short myTicks = 0;
	short didTimeOutOccur = FALSE;

	if(myPort == COMM_0 || myPort == COMM_1)
	{                  		
		while(*myBuf)
			putchar(myPort, *myBuf++);
	}
	else if(myPort == USB_PORT) //Fill buffer only
	{
		//Append prompt and short multi-line responses but do not initiate transfer.
		//Previously, the reply and the prompt each initiated transfers which is SLOW!	
			
		while(*myBuf)
		{
			if(myCount < 61)
			{
				myUsbInTmpBuf2[myCount++] = *myBuf++;
			}
			else//Buffer full, transmit it now
			{
				break;
			}
		}
		if(myCount>60)
		{
			myUsbInTmpBuf2[myCount] = 0; //Terminate the string	
			wDogKick();		
			while(UsbInPending == TRUE)
			{					
				if(myTicks > MAX_USB_TIMEOUT_IN_TICKS) //200 ticks = 1 second
				{
					swError(__FILE__, __LINE__);
					didTimeOutOccur = TRUE;
					break;
				}
				else
				{
					delayMs(MAX_TIMEOUT_USB); //5ms
					myTicks++;
				}		
			}				
			if(didTimeOutOccur == FALSE)
			{	
				memcpy(myUsbInTmpBuf,myUsbInTmpBuf2,64); //Now that we have the lock, write to the buffer shared with interrupt
				UsbInPending = TRUE;				
				myCount = 0;									
			}
			else
			{								
				UsbInPending=FALSE;
				myCount = 0;
			}
			memset(myUsbInTmpBuf2,0x00,64);					
		}		
	}
	else if(myPort == USB_PORT_INIT) //Initiate a transfer
	{
		wDogKick(); //For large transfers, like help file					 
		while(UsbInPending == TRUE)
		{				
			if(myTicks > MAX_USB_TIMEOUT_IN_TICKS) //200 ticks = 1 second
			{
				swError(__FILE__, __LINE__);
				didTimeOutOccur = TRUE;
				break;
			}
			else
			{
				delayMs(MAX_TIMEOUT_USB); //5ms
				myTicks++;
			}		
		}
			
		if(didTimeOutOccur == FALSE)
		{	
			while(*myBuf)
			{
				if(myCount < 61)
				{
					myUsbInTmpBuf2[myCount++] = *myBuf++;
				}
				else
				{
					break;
				}
			}			
				
			if(myCount > 0)
			{
				myUsbInTmpBuf2[myCount] = 0; //Terminate the string
				memcpy(myUsbInTmpBuf,myUsbInTmpBuf2,64);
				memset(myUsbInTmpBuf2,0x00,64);
				myCount = 0;
				UsbInPending = TRUE;								
			}
		}
		else
		{
			myCount = 0;
			UsbInPending=FALSE;
		}
	}
	else
	{
		swError(__FILE__, __LINE__);
	}
}

int getchar1 (void) 				/* Read character from Serial Port   */
{                    

  while (!(U1LSR & 0x01));

  return (U1RBR);
}

int putcharComm1 (int ch) 			  // Write character to Serial Port    
{                  		

#ifdef UART1_INT										
  	while (!(U1LSR & 0x20));
    	return (U1THR = ch);
#else
	if(wr1OutBuf > MAX_SERIAL_BUF_SIZE)							
		wr1OutBuf = 0;
 
 	tx1Buffer[wr1OutBuf++] = ch;

/*	if(U1LSR & THR_EMPTY)
	{
		if(rd1OutBuf > MAX_SERIAL_BUF_SIZE)
			rd1OutBuf = 0;
		if((rd1OutBuf != wr1OutBuf) )
			U1THR = tx1Buffer[rd1OutBuf++];
	}		*/
	return PASS;
 #endif 

}


short getCharComm1 (char* myChar) 			/* Read character from Serial Port   */
{
	short myStatus = PASS;
	                   
	VICIntEnClr = 1 << 7;       /* Disable UART1 Interrupt */

	if( wr1InBuf > rd1InBuf)
	{
		*myChar = rx1Buf[rd1InBuf++];
	}
	else if( rd1InBuf > MAX_SERIAL_BUF_SIZE)
	{
		rd1InBuf = 0;

		if(wr1InBuf > rd1InBuf)
			*myChar = rx1Buf[rd1InBuf++];
	}
	else
		myStatus = FAIL;

  VICIntEnable = 1 << 7;       /* Enable UART1 Interrupt */
	
	return myStatus;	
}


//uart0 isr
void uart0_isr (void) __irq
{
	uchar myChar;
	//uchar myCnt;

	if(U0IIR & RX_LINE_STATUS)
	{
		if(U0LSR & OVERRUN_ERR)
			swTrace(__FILE__, __LINE__);		// log error COM0 overrun error!
		else if(U0LSR & FRAMING_ERROR)
			swTrace(__FILE__, __LINE__);		// log error COM0 framing error!
		else if (U0LSR & RX_ERROR)
			swTrace(__FILE__, __LINE__);		// log error "COM0 RX error!
		else
		{
			while(U0LSR & RX_DATA_RDY)
			{
				myChar = U0RBR;

				wasHere = 0x20;

				if(mySerial0Pos > MAX_SERIAL_BUF_SIZE)
					mySerial0Pos = 0;
				mySerial0In[mySerial0Pos++] = myChar;


#if 0
				comm0RxBuf[comm0RxInPos++] = myChar;

				if(comm0RxInPos	> MAX_CMD_SIZE)		// flush the buffer?
				{
					comm0RxInPos = 0;

					for(myCnt = 0; myCnt < MAX_CMD_SIZE; myCnt++)
						comm0RxBuf[comm0RxInPos++] = 0;
				}

				if(myChar == CR_KEY)
				{
					charsToProcessRx0 = comm0RxInPos;

					for(myCnt = 0; myCnt < comm0RxInPos; myCnt++)
					{
						rx0Buf[myCnt] = comm0RxBuf[myCnt];
						comm0RxBuf[myCnt] = 0;
					}

					comm0RxInPos = 0;
				}
#endif
				//writeQ(SER0_Q, IN_Q, &myChar, 1);

				wasHere = 0x21;

				if(sysCfgShadowRegs.echo == TRUE)	// echo/send the character back?
				{
					 while (!(U0LSR & 0x20));		 // FRG FIX!!!
	  				U0THR = myChar;
						wasHere = 0x22;
				}
#if 0						   
				if(wr0InBuf == rd0InBuf)
				{
					rd0InBuf = 0;
					wr0InBuf = 0;
				}
				else if(wr0InBuf > MAX_SERIAL_BUF_SIZE)
					wr0InBuf = 0;
				
				//rx1Buf[wr1InBuf++] = U1RBR;
				rx0Buf[wr0InBuf++] = myChar;
#endif			 	 
			}
		}
	}
	else if(U0IIR & CHAR_TIMEOUT)
	{
		myChar = U0RBR;
		//writeQ(SER0_Q, IN_Q, &myChar, 1);

		//if(sysCfgShadowRegs.echo == TRUE)	// echo/send the character back?
		//{
			 //while (!(U0LSR & 0x20));
	  		//U0THR = myChar;
		//}

#if 0
		if(wr0InBuf == rd0InBuf)
		{
			rd0InBuf = 0;
			wr0InBuf = 0;
		}
		else if(wr0InBuf > MAX_SERIAL_BUF_SIZE)
			wr0InBuf = 0;
		
		rx0Buf[wr0InBuf++] = U1RBR;
#endif			 		
	}
	else if(U0IIR & RX_DATA_AVAIL )
	{

		while(U0LSR & RX_DATA_RDY)
		{
			myChar = U0RBR;
			//writeQ(SER0_Q, IN_Q, &myChar, 1);

			//if(sysCfgShadowRegs.echo == TRUE)	// echo/send the character back?
			//{
				 //while (!(U0LSR & 0x20));
	  			//U0THR = myChar;
			//}
									   
#if 0
			if(wr0InBuf == rd0InBuf)
			{
				wr0InBuf = 0;
				rd0InBuf = 0;
			}
			else if(wr0InBuf > MAX_SERIAL_BUF_SIZE)
					wr0InBuf = 0;
			
			rx0Buf[wr0InBuf++] = U0RBR;	
#endif		 	 
		}
		
	}
		  
	VICVectAddr = 0;                 /* Acknowledge Interrupt */	   
}


//uart1 isr
void uart1_isr (void) __irq
{
	uchar myChar;

	wasHere = 0x120;

	if(U1IIR & RX_LINE_STATUS)
	{
		if(U1LSR & OVERRUN_ERR)
			swError(__FILE__, __LINE__);		// log error COM1 overrun error!
		else if(U1LSR & FRAMING_ERROR)
			swError(__FILE__, __LINE__);		// log error COM1 framing error!
		else if (U1LSR & RX_ERROR)
			swError(__FILE__, __LINE__);		// log error "COM1 RX error!
		else
		{
			while(U1LSR & RX_DATA_RDY)
			{
				myChar = U1RBR;
				writeQ(SER1_Q, IN_Q, &myChar, 1);

				if(sysCfgShadowRegs.echo == TRUE)	// echo/send the character back?
				{
					 while (!(U1LSR & 0x20));
	  				U1THR = myChar;
				}
#if 0						   
				if(wr1InBuf == rd1InBuf)
				{
					rd1InBuf = 0;
					wr1InBuf = 0;
				}
				else if(wr1InBuf > MAX_SERIAL_BUF_SIZE)
					wr1InBuf = 0;
				
				//rx1Buf[wr1InBuf++] = U1RBR;
				rx1Buf[wr1InBuf++] = myChar;
#endif			 	 
			}
		}
	}
	else if(U1IIR & CHAR_TIMEOUT)
	{
		myChar = U1RBR;
		writeQ(SER1_Q, IN_Q, &myChar, 1);

		if(sysCfgShadowRegs.echo == TRUE)	// echo/send the character back?
		{
			 while (!(U1LSR & 0x20));
	  		U1THR = myChar;
		}

#if 0
		if(wr1InBuf == rd1InBuf)
		{
			rd1InBuf = 0;
			wr1InBuf = 0;
		}
		else if(wr1InBuf > MAX_SERIAL_BUF_SIZE)
			wr1InBuf = 0;
		
		rx1Buf[wr1InBuf++] = U1RBR;
#endif			 		
	}
	else if(U1IIR & RX_DATA_AVAIL )
	{

		while(U1LSR & RX_DATA_RDY)
		{
			myChar = U1RBR;
			writeQ(SER1_Q, IN_Q, &myChar, 1);

			if(sysCfgShadowRegs.echo == TRUE)	// echo/send the character back?
			{
				 while (!(U1LSR & 0x20));
	  			U1THR = myChar;
			}
									   
#if 0
			if(wr1InBuf == rd1InBuf)
			{
				wr1InBuf = 0;
				rd1InBuf = 0;
			}
			else if(wr1InBuf > MAX_SERIAL_BUF_SIZE)
					wr1InBuf = 0;
			
			rx1Buf[wr1InBuf++] = U1RBR;	
#endif		 	 
		}
		
	}
		  
	VICVectAddr = 0;                 /* Acknowledge Interrupt */	   
}


#if 0
int getkey (uchar myPort)                      // Read character from Serial Port 1  
{
	uchar myChar;

	if(myPort == COMM_0)
	{
		if((U0LSR & 0x01) == 1)
		{
			myChar = U0RBR;
			writeQ(SER0_Q, IN_Q, &myChar, 1);
			mySerial0In[mySerial0Pos] = myChar;
		}
		else
			return (0);

		if((mySerial0Pos + 1) >= MAX_SERIAL_BUF_SIZE)
			mySerial0Pos = 0; 

		return (mySerial0In[mySerial0Pos++]);
	}
	else
	{
		if((U1LSR & 0x01) == 1)
		{
			myChar = U1RBR;
			writeQ(SER1_Q, IN_Q, &myChar, 1);
			mySerial1In[mySerial1Pos] = myChar;
		}
		else
			return (0);

		if((mySerial1Pos + 1) >= MAX_SERIAL_BUF_SIZE)
			mySerial1Pos = 0;
			
		return (mySerial1In[mySerial1Pos++]); 
	}
			
}
#endif

