/////////////////////////////////////////////////////////////////////////////
//
//						sysCfg.h
//
//	System Configuration attributes
//
//	(C) SoftwareFlair Inc. 2008
//
//
//
/////////////////////////////////////////////////////////////////////////////

#define MAX_LASER_ID	40

// System Configuration Defaults.
#define EE_DEF_SYSCFG_VERSION					0x01
#define EE_DEF_LP								0
#define EE_DEF_C								0
#define EE_DEF_PROTO_VORT				0
#define EE_DEF_LSR_HOURS				0.0
#define EE_DEF_LC								0
#define EE_DEF_OBT_DEV					5	// 5 degree deviation
#define EE_DEF_CAL_DAC					0
#define EE_DEF_EPC							0
#define EE_DEF_DELAY						1
#define EE_DEF_VOBTS						22
#define EE_DEF_TEC_ON						1
#define EE_DEF_ECHO							1
#define EE_DEF_PROMPT						1
#define EE_DEF_PEAK_PWR					0
#define EE_DEF_PULSE_MODE				0
#define EE_DEF_LSR_ID						"VL0000000, 10010, 405nm, 100mW, C"
#define EE_DEF_CURR_DRV                 4095
#define EE_DEF_DFBLC 					0
#define EE_DEF_HTR_CNT					0
#define EE_DEF_CAL_DAC_DFB              0
#define EE_DEF_DFBLC_DAC				0


// Calibration Defaults.
#define EE_DEF_CAL_VERSION			0x03
#define EE_CAL_VERSION				0.0
//#define EE_MAX_PWR_LSR				1000
#define EE_MAX_PWR_LSR				200
#define EE_INIT_LSR_CUR				150
#define EE_INIT_DFB_CUR			150
#define EE_NOMINAL_LSR_PWR		100
#define EE_MAX_LSR_CUR				500
#define EE_NOMINAL_WAVE_LEN		640
#define EE_MEAS_WAVE_LEN			641
#define EE_LSR_PWR_SET_POT		0x00		// should never write these!!
#define EE_LSR_CUR_SET_POT		0x00		// should never write these!!
#define EE_LSR_PART_NUM				"00000"
#define EE_LSR_SERIAL_NUM			"VLmmyy####"
#define EE_LSR_BEAM_SHAPE			"E"
#define EE_LC_SCALE_FACTOR      100  //defaults to 100 to allow 1/100 resolution
#define EE_EPC_SCALE_FACTOR     100  //defaults to 100 to allow 1/100 resolution
#define EE_LP_OFFSET_ADJUST       0  //defaults to   0 and allows 1 unit resolution
#define EE_END_OF_LIFE            1  //defaults to Enabled
#define EE_EOL_FAULT_LINE         1  //defaults to Enabled
#define EE_DEF_NVPUL              0  //defaults to Disabled
#define EE_EPC_SCALE_FACTOR2    100  //defaults to 100 to allow 1/100 resolution
#define EE_MODE                   0  //defaults to 0 "Standard Product"
#define EE_RFINJ                  0  //defaults to Disabled
#define EE_DFB_MAX_CURR 		  200
#define EE_HTR_MAX_CURR 		  200
#define EE_DEF_MAX_OVER_CURR 	500

// EEPROM image areas
typedef enum {
	EE_AREA_SYS_CFG = 0,
	EE_AREA_CAL,
	EE_AREA_HELP,
	EE_AREA_SW_ERR,
	EE_AREA_END				// DO NOT REMOVE!! USED TO specify the last entry
} EE_AREA_TYPE;

// this system configuration shadow registers MUST match the locations and sizes of the "EEPROM Memory Image"!
typedef struct {
	ushort 		version;
	ushort		vobts;
	float  		lp;
	float 		lc;
	float		laserHours;
	uchar  		c;
	uchar		protoMode;
	uchar		epc;
	uchar		delay;
	uchar		isTecOn;
	uchar		echo;
	uchar		prompt;
	ushort	    peakPower;
	uchar		pulseMode;
	ushort      currDrvSetting;
	float       dfblc;
	ushort 		dfblcDacCnt;
} SYS_CFG_SHADOW_REG;

// this system configuration shadow registers MUST match the locations and sizes of the "EEPROM Memory Image"!
typedef struct {
	ushort		version;
	float       calVersion;
	float 		maxPwrLsr;
	short		initLsrCur;
	short       initDfbCur;
	short		nominalLsrPwr;
	short		maxLsrCur;
	short		nominalWaveLen;
	short		measWaveLen;
	short		lsrPwrSetPot;
	short		lsrCurSetPot;
	char		lsrPartNum[6];
	char		lsrSerialNum[11];
	char		lsrBeamShape[4];
	uchar		obtDeviation;
	short		calDacValue;
	short       lcScaleFactor;
	short       epcScaleFactor;
	short       lpOffsetAdjust;
	short       endOfLifeEnable;
	short       endOfLifeFaultLine;
	short       nonVolPUL;
	short       epcScaleFactor2;
	short       mode;
	short       rfInj;
	short 		maxCurrDfb;
	short		maxCurrHtr;
	ushort		htrCnt;
	ushort      calDacDfbValue;
	ushort      maxOverCurr;
} CAL_SHADOW_REG;

typedef struct {
	char 	helpStr1[10];
} HELP_SHADOW_REG;

extern short isCalImageGood;		 // if the Calibration Image is corrupted then do not allow the laser to be turned on!

extern void sysCfgInit(void);
extern void saveSysCfgDelta(EE_AREA_TYPE);
extern SYS_CFG_SHADOW_REG sysCfgShadowRegs;
extern CAL_SHADOW_REG calShadowRegs;
extern HELP_SHADOW_REG helpShadowRegs;
