/*----------------------------------------------------------------------------
 *      U S B  -  K e r n e l
 *----------------------------------------------------------------------------
 *      Name:    USBDESC.C
 *      Purpose: USB Descriptors
 *      Version: V1.03
 *----------------------------------------------------------------------------
 *      This file is part of the uVision/ARM development tools.
 *      Copyright (c) 2005 Keil Software. All rights reserved.
 *---------------------------------------------------------------------------*/

#include "type.h"

#include "usb.h"
#include "usbcfg.h"
#include "usbdesc.h"


/* USB Standard Device Descriptor */
const BYTE USB_DeviceDescriptor[] = {
  USB_DEVICE_DESC_SIZE,                 /* bLength */
  USB_DEVICE_DESCRIPTOR_TYPE,           /* bDescriptorType */
  WBVAL(0x0110), /* 1.10 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USB_MAX_PACKET0,                      /* bMaxPacketSize0 */
  WBVAL(0x0A00),                        /* idVendor */	// FRG TODO: Vortran needs a VID and PID
  WBVAL(0x0001),                        /* idProduct */
  WBVAL(0x0100), /* 1.00 */             /* bcdDevice */
  0x04,                                 /* iManufacturer */
  0x14,                                 /* iProduct */
  0x44,                                 /* iSerialNumber */
  0x01                                  /* bNumConfigurations */
};



/* USB Configuration Descriptor */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor */
const BYTE USB_ConfigDescriptor[] = {
/* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(                                /* wTotalLength */
    USB_CONFIGURATION_DESC_SIZE         +
    USB_INTERFACE_DESC_SIZE             
  ),
  0x01,                                 /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue */
  0x00,                                 /* iConfiguration */
  USB_CONFIG_BUS_POWERED,               /* bmAttributes */
  USB_CONFIG_POWER_MA(100),             /* bMaxPower */
/* Interface 0, Alternate Setting 0, Audio Control */
  USB_INTERFACE_DESC_SIZE,              /* bLength */
  USB_INTERFACE_DESCRIPTOR_TYPE,        /* bDescriptorType */
  0x00,                                 /* bInterfaceNumber */
  0x00,                                 /* bAlternateSetting */
  0x00,                                 /* bNumEndpoints */
  USB_DEVICE_CLASS_VENDOR_SPECIFIC,               /* bInterfaceClass */
  0xFF,          /* bInterfaceSubClass */
  0x00,             /* bInterfaceProtocol */
  0x00,                                 /* iInterface */
/* Terminator */
  0                                     /* bLength */
};

/* USB String Descriptor (optional) */
const BYTE USB_StringDescriptor[] = {
/* Index 0x00: LANGID Codes */
  0x04,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  WBVAL(0x0409), /* US English */    /* wLANGID */
/* Index 0x04: Manufacturer */
  0x0E,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'V',0,
  'o',0,
  'r',0,
  't',0,
  'r',0,
  'a',0,
  'n',0,
/* Index 0x14: Product */
  0x30,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'V',0,
  'o',0,
  'r',0,
  't',0,
  'r',0,
  'a',0,
  'n',0,
  ' ',0,
  'S',0,
  't',0,
  'a',0,
  'd',0,
  'u',0,
  's',0,
  ' ',0,
  '',0,
  '2',0,
  '0',0,
  '0',0,
  '8',0,
/* Index 0x44: Serial Number */
  0x18,                              /* bLength */
  USB_STRING_DESCRIPTOR_TYPE,        /* bDescriptorType */
  'V',0,
  'o',0,
  'r',0,
  't',0,
  'r',0,
  'a',0,
  'n',0,
};
