/*----------------------------------------------------------------------------
 *      Name:    wdog
 *      Purpose: Watch Dog
 * 
 *	WatchDog  if the system is not continually running in the super loop, then the 
 *	watch dog shall not be kicked, and the watch dog handler shall be called. The 
 *	watch do handler is responsible for saving bread crumbs, and restarting the
 *	system with a warm boot     
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <LPC214X.H>                        /* LPC214x definitions */
#include "system.h"
#include "type.h"


#define WDTOF	0x04		// Watch Dog Timeout Flag

extern void reset(void);

ushort wDogTimeOut = 0x2000;
ushort resetSrcId = 0;
uchar wDogActive = 1;
int wDogFeedState = 0;
int myVicIntReg = 0;
int prevWDogOccurred = FALSE;

void wDogKick(void);

// Watch Dog Interrupt Service Routine
void wDog_isr (void) __irq
{
	VICIntEnClr = 1 << 0;                    	// Disable Watch Dog Interrupt
	VICVectAddr = 0;                          // Acknowledge Interrupt 
	
	//WDMOD = 0x2;	 	// Reset


	allInterruptsOff();
	reset();
}
																 
void wDogInit(void)
{

	if(WDMOD & WDTOF)		// did we just get reset from the watchdog?
		prevWDogOccurred = TRUE;		 // yes

	// Set the watchdog timer constant reload value in WDTC register.
	WDTC = 0x8000000;	// ~8 seconds

	// Start the watchdog by writing 0xAA followed by 0x55 to the WDFEED register.
	wDogKick();

	// Setup mode in WDMOD register.
	WDMOD = 0x3;	//Watchdog Reset Mode: debug with the Watchdog reset, no interrupt
								// WDRESET enabled.	When this mode is selected, a watchdog counter 
								// underflow will set the	WDINT flag and the watchdog interrupt request
								// will be generated.

	// SetupWatch Dog Interrupt 
	VICVectAddr0 = (unsigned long)wDog_isr;	// wDog Interrupt -> Vector 0
	VICVectCntl0 = 0x20 | 0;				// wDog Interrupt -> IRQ Slot 0 


	VICIntEnable = 1 << 0;                    	// Enable Watch Dog Interrupt

	// Note: Watchdog should be fed again before the watchdog counter underflows to prevent reset/interrupt

}

void wDogKick(void)
{
	if(wDogActive)
	{
		wDogKickSupr();
	}
}

void wDogTimedOut(void)	//record issue with EEPROM, send error msg, restart system
{

}

void wDogSetTimeOut(void)
{
}

void wDogGetTimeOut(void)
{
}


// This register contains one bit for each source of Reset. 
void resetSourceId(void)
{
	resetSrcId = RSID;

	if(RSID	| 0x01)		//POR
		;// "Power On Reset"

	if(RSID	| 0x02)		//EXTR
		;// "Assertion of the Reset line"

	if(RSID	| 0x04)	 	//WDTR
		;// Watch Dog Timeout

	if(RSID	| 0x08)		//BODR
		;//"Brown Out: Power dip below 2.6v"
}

// This register contains one bit for each source of Reset. Writing a 1 to any of these bits
// clears the corresponding read-side bit to 0.
void resetSourceIdClr(void)
{
	RSID = 0x0F;	 // clear POR, EXTR, WDTR, BODR
}

